/*
 * This program uses a simple hashing algorithm on C-strings.  It hashes
 * all of the lines of text in a file, finds the lines with the minimum
 * and maximum hash values, and then checks to see what happens to the
 * hash values when those lines are doubled (concatenated with themselves.)
 * Sadly, this segfaults.
 */

#include <iostream>
#include <fstream>
#include <string.h>

#include "strhash.h"

using namespace std;

const int NUM_LINES = 20;

int main (int argc, char** argv)
{
  // Note that we need an argument from the command line
  if (argc < 2)
    {
      cout << "Requires name of text file to operate on." << endl;
      exit(0);
    }
  
  // Get a stream for reading our input file
  ifstream inFile(argv[1]);

  // If opening our stream didn't work out, error and exit.
  if (inFile.bad())
    {
      cout << "Unable to open " << argv[1] << " for reading." << endl;
      exit(0);
    }

  // Get an array of lines to operate on
  StrHash* lines = new StrHash[NUM_LINES];
  int curLine = 0;
  string temp;
  int maxVal = -1;
  int maxInd = 0;
  int minVal = -1;
  int minInd = 0;

  // Read until EOF or we have read in all the lines we want
  getline(inFile, temp);
  while (curLine < NUM_LINES && inFile)
    {      
      lines[curLine] = *(new StrHash(temp.c_str(), strlen(temp.c_str())));

      if (lines[curLine].getHash() > maxVal || maxVal == -1) 
	{
	  maxVal = lines[curLine].getHash();
	  maxInd = curLine;
	} 
      if (lines[curLine].getHash() < minVal || minVal == -1)
	{
	  minVal = lines[curLine].getHash();
	  minInd = curLine;
	}

      getline(inFile, temp);
      curLine++;
    }
  
  // Now we can get real copies of the min and max hashes.
  StrHash min = lines[minInd];
  StrHash max = lines[maxInd];

  // And free up our memory
  delete lines;

  cout << "Before the doubling operation, min & max are:" << endl;
  cout << min << endl;
  cout << max << endl;
  cout << endl;

  // Now concatenate twice if possible
  char tempBuf[BUFFER_SIZE];
  strncat(tempBuf, min.getBuf(), BUFFER_SIZE);
  strncat(tempBuf, min.getBuf(), BUFFER_SIZE);
  min.store(tempBuf, strlen(tempBuf));
  tempBuf[0] = 0;
  
  strncat(tempBuf, max.getBuf(), BUFFER_SIZE);
  strncat(tempBuf, max.getBuf(), BUFFER_SIZE);
  max.store(tempBuf, strlen(tempBuf));

  cout << "After doubling, min & max are:" << endl;
  cout << min << endl;
  cout << max << endl;
  cout << endl;

}
