#include <iostream>
#include <stdio.h>
#include "strhash.h"

using namespace std;

StrHash::StrHash(const char* _buf)
{
  buf = new char[BUFFER_SIZE];
  memcpy(buf, _buf, BUFFER_SIZE);

  rehash();
}

StrHash::StrHash(const char* _buf, int n)
{
  if (n > BUFFER_SIZE) n = BUFFER_SIZE;

  buf = new char[BUFFER_SIZE];
  memcpy(buf, _buf, n);

  rehash();
}

StrHash::StrHash()
{
  buf = new char[BUFFER_SIZE];

  rehash();
}


StrHash::~StrHash()
{
  delete buf;
}

void StrHash::store(const char* _buf, int n)
{
  if (n > BUFFER_SIZE) n = BUFFER_SIZE;

  memcpy(buf, _buf, n);
  rehash();
}

unsigned int StrHash::rehash()
{
  unsigned int ret = 0;

  // Loop through the buffer
  for (int i = 0; i < BUFFER_SIZE; i++)
    {
      // Get the 5 left-most bits in our counter
      unsigned int temp = ret >> 27;

      // Wrap the counter 5 bits to the left
      ret = ret << 5;

      // Add in the 5 bits we just pushed off the left edge.
      ret |= temp;

      // And finally, add in the new byte
      ret ^= (unsigned char)buf[i];
      
    }

  ret = ret * (ret + 3);

  hashVal = ret;
  return ret;
}

unsigned int StrHash::getHash()
{
  return hashVal;
}

const char* StrHash::getBuf()
{
  return buf;
}

ostream& operator << (ostream& targetStream, const StrHash& obj)
{
  // We only print out the hash value because the rest of the information
  // in the object may not be printable, and generally we are only
  // interested in the hash values anyway.
  targetStream << obj.hashVal;  

  return targetStream;
}
