#ifndef STRHASH_H
#define STRHASH_H

#include <iostream.h>

// How large a buffer are we supporting?
const unsigned int BUFFER_SIZE = 512;

class StrHash
{
  // So we can print cleanly
  friend ostream& operator << (ostream& targetStream, const StrHash& sh); 

 public:
  // Default constructor, fills buffer with 0s
  StrHash();

  // Copies BUFFER_SIZE bytes from _buf.  
  // This is DANGEROUS because we don't have guarentees that _buf is 
  // actually that long.  But then, we aren't writing into _buf so
  // it (probably) won't segfault or anything.
  StrHash(const char* _buf);

  // Takes in a buffer and a length, copies min(n, BUFFER_SIZE) bytes into
  // buf.
  StrHash(const char* _buf, int n);

  // Destructor
  ~StrHash();

  // Copy min(n, BUFFER_SIZE) bytes of data into the buffer for this object.
  void store(const char* _buf, int n);

  // This function will recalculate the hash value of the object
  // based on the current data in the buffer.
  unsigned int rehash();

  // This function will return the last calculated hash value,
  // which is quicker than recalculating it each time.
  unsigned int getHash();

  // This returns a const pointer to the data region for this object
  const char* getBuf();

 private:
  // This is the data of the object
  char* buf;

  // This is the last calculated hash value
  unsigned int hashVal;
};

ostream& operator << (ostream& targetStream, const StrHash& sh); 

#endif // STRHASH_H
